;
; REACT - Run EAC Tasks
; AutoIt3.1.1 script - http://www.autoitscript.com
;
; ----------------------------------------------------------------------------------------------------
; Copyright (C) Tycho, 2006
;
; REACT is provided 'as-is', without any express or implied warranty.
; In no event will the authors be held liable for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any purpose,
; including commercial applications, and to alter it and redistribute it freely,
; subject to the following restrictions (known as zlib license):
;
;   1. The origin of this software must not be misrepresented; you must not claim that you wrote
;      the original software. If you use this software in a product, an acknowledgment in the product
;      documentation would be appreciated but is not required.
;   2. Altered source versions must be plainly marked as such, and must not be misrepresented as being
;      the original software.
;   3. This notice may not be removed or altered from any source distribution.
; ----------------------------------------------------------------------------------------------------
;
; REACT Documentation - http://www.hydrogenaudio.org/forums/index.php?showtopic=35158
; EAC                 - http://www.exactaudiocopy.org

#NoTrayIcon

Opt("MustDeclareVars", 1)
Opt("RunErrorsFatal", 0)

; Include GUI contants and additonal meta data code
#include <GUIConstants.au3>
#include "REACT Additonal Meta Data.au3"
#include "REACT Start From Cuesheet.au3"

Global $g_version = "2.0.ssb16p"
Global $g_imageExt = "wav"
Global $g_appName = StringTrimRight(@ScriptName, 4)
Global $g_appTitle = $g_appName & " v" & $g_version
Global $g_iniFile = @ScriptDir & "\" & $g_appName & ".ini"
Global $g_minCompWindow = 1
Global $g_processPriority = 1

Global $g_eacExe = RegRead("HKEY_CURRENT_USER\Software\AWSoftware\EAC", "InstallPath") & "EAC.exe"
Global $g_myMusicDir = RegRead("HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders", "My Music")
Global $g_toolsDir = @ScriptDir & "\tools"
Global $g_coverDownloaderExe = @ScriptDir & "\coverdownloader\albumart.exe"
Global $g_runCoverDownloader = 1
Global $g_cover = ""

Global $g_eacTitle = "Exact Audio Copy"
Global $g_eacText = "CD Artist"
Global $g_eacHandle = 0
Global $g_eacExtrAudioTitle = "Extracting Audio Data"
Global $g_eacExtrDoneMinTitle = "[100.0%]    Extracting"
Global $g_imageHotKey = "{F10}"
Global $g_tracksHotKey = "{F4}"
Global $g_tracksHotVal = "+{F5}"
Global $g_VA = "Various Artists"

Global $g_imgMenu = StringSplit("&Action|Copy &Image && Create CUE Sheet|Compressed...", "|")
Global $g_saveWaveformStr = "Save Waveform"
Global $g_imageNaming = "$artist$ - [$year$] $album$"
Global $g_convKey = "Sla_Bks_Col_Qst_Bar_Quo_Ast_Lt_Gt"
Global $g_convString = "-|-|-||!|'|#|[|]"
Global $g_convTable

Global $g_source_tmp
Global $g_destimage
Global $g_cuesheet
Global $g_sourcecuesheet
Global $g_albumfile = "{undefined}"
Global $g_batfile
Global $g_eaclog
Global $g_sourcedir
Global $g_basename
Global $g_various = 0

Global $g_cueMenu = StringSplit("&Action|Create CUE &Sheet|Multiple WAV Files With Gaps...  (Noncompliant)", "|")

Global $g_cdartist, $g_artist, $g_album, $g_title, $g_genre, $g_freedb, $g_comment
Global $g_cdartist_fn, $g_artist_fn, $g_album_fn, $g_track_fn, $g_numtracks_fn, $g_title_fn, $g_genre_fn, $g_freedb_fn, $g_comment_fn
Global $g_track, $g_numtracks, $g_year, $g_freedbID, $g_crc, $g_bitrate

Global $gui_album, $gui_artist, $gui_year, $gui_genre, $gui_freedb, $gui_albumfile
Global $g_userTrackFormats = 0, $g_userOutputNames = 0, $g_userSettings = 0
Global $g_trackName, $g_logfile
Global $g_debugging = false
Global $g_coverDownloaderParams, $g_coverDownloaderXUI
Global $g_coverDownloaderLocalPath
Global $gui_albumfile2, $g_directory, $g_infocuesheet

fMain()
Exit


Func fMain()
	Local $appid = "REACT-847323"

	$g_eacExe = IniRead($g_iniFile, "Settings", "EAC", $g_eacExe)
	$g_convString = IniRead($g_iniFile, "Settings", $g_convKey, $g_convString)
	$g_convTable = StringSplit($g_convString, "|")

	If $CmdLine[0] > 0 Then
		If $CmdLine[0] = 1 Then
			fStartFromCuesheet()
		Else
			fEncode()
		EndIf
	ElseIf Not WinExists($appid) Then
		Opt("TrayIconHide", 0) ; Show tray icon
		AutoItWinSetTitle($appid)
		fMainLoop()
	Else
		Local $pid = Run($g_eacExe)
		If @error = 1 Then
			MsgBox(0+48+8192, $g_appTitle, "Couldn't start " & $g_eacExe)
			Exit
		EndIf
		WinWait($g_eacTitle, $g_eacText)
	EndIf
EndFunc


Func fMainLoop()

	; Are we in debug mode?
	$g_debugging = Int(IniRead($g_iniFile, "UserSettings", "DebugMode", "0"))

	; Handle old ini file.
	If FileExists($g_iniFile) Then
		Local $ver = IniRead($g_iniFile, "Settings", "Version", 0.0)
		Local $arrVersion = StringSplit($g_version, ".")
		Local $arrIniVersion = StringSplit($ver, ".")
		fDebugPrint("Version: " & $g_version)
		fDebugPrint("INI version: " & $ver)
		If UBound($arrVersion) < 2 Then ReDim $arrVersion[2]
		If $arrVersion[1] <> $arrIniVersion[1] Or $arrVersion[2] <> $arrIniVersion[2] Then
			fDebugPrint("Version change.  Rewriting INI file")
			fReadIni()
			FileDelete($g_iniFile)
			fWriteIni()
			; No need to reconfigure EAC - no changes.
		EndIf
	EndIf

	If Not FileExists($g_iniFile) Then
		fDebugPrint("INI file not found.  Writing new file")
		fWriteIni()
		fConfigEAC()
	EndIf

	; Check naming scheme
	Local $p = "HKEY_CURRENT_USER\Software\AWSoftware\EAC\Extraction Options"
	Local $e = "FileNamingConvention"
	If RegRead($p, "UseVariousFileNamingConvention" ) = "FF" Then $e = "VariousFileNamingConvention"
	Local $c = RegRead($p, $e)
	If StringLeft($c, 2) <> "%D" Or StringInStr($c, "\") > 0 Then
		MsgBox(0+48+8192, $g_appTitle, "EAC active naming scheme must start with '%D' and not contain '\'" & @crlf & _
					       "Please, press Ctrl-F2 to configure EAC for use with REACT")
	EndIf
	fDebugPrint($e & ": " & $c)
	$c = RegRead("HKEY_CURRENT_USER\Software\AWSoftware\EAC\StartUp Options", "UseLanguage" )
	If @error = 0 And $c <> "47AB3DF2" Then
		MsgBox(0+48+8192, $g_appTitle, "EAC must use English language for REACT to work." & @crlf & _
					       "Please, change language settings to English")
	EndIf

	; ---------------- run EAC ---------------------

	Opt("WinTitleMatchMode", 2)

	If Not (WinExists($g_eacTitle, $g_eacText) Or WinExists("]    Extracting") Or WinExists("]    Detecting Gaps")) Then
		Local $pid = Run($g_eacExe)
		If @error = 1 Then
			MsgBox(0+48+8192, $g_appTitle, "Couldn't start " & $g_eacExe)
			Exit
		EndIf
		WinWait($g_eacTitle, $g_eacText)
	EndIf

	$g_eacHandle = WinGetHandle($g_eacTitle, $g_eacText)

	fReadIni()

	HotKeySet("!{F10}", "fGUIRunCoverDownloader")

	Local $hot = 0
	Local $ok = 1
	
	; Create the GUI
	fDiagCreate()
	
	While $ok
		If WinActive($g_eacTitle, $g_eacText) Then
			If Not $hot Then
				$hot = 1
				HotKeySet($g_imageHotKey, "fImageCopyCompressed")
				HotKeySet($g_tracksHotKey, "fTracksCopyCompressed")
				HotKeySet("{F1}", "fHelpEAC")
				HotKeySet("!{F1}", "fHelp")
				HotKeySet("!{F2}", "fShowIni")
				HotKeySet("^{F2}", "fConfigEAC")
				HotKeySet("!{F3}", "fExploreConfig")
				HotKeySet("!{F5}", "fDiagOpen") ; Open Additional Meta Data dialogue
				fSetEACTitle()
			EndIf
		ElseIf $hot Then
			$hot = 0
			HotKeySet($g_imageHotKey)
			HotKeySet($g_tracksHotKey)
			HotKeySet("{F1}")
			HotKeySet("!{F1}")
			HotKeySet("!{F2}")
			HotKeySet("^{F2}")
			HotKeySet("!{F3}")
			HotKeySet("!{F5}")
		EndIf
		
		; Check whether the main window title has changed
		fCheckEACTitleChange()

		If WinExists($g_eacExtrDoneMinTitle) Then
			; Restore if minimized
			If BitAnd(WinGetState($g_eacExtrDoneMinTitle), 16) <> 0 Then WinSetState($g_eacExtrDoneMinTitle, "", @SW_RESTORE)
		EndIf

		Sleep(250)

		If WinExists($g_eacExtrAudioTitle, "OK") And Not WinExists($g_eacExtrAudioTitle, "Cancel") Then
			ControlClick($g_eacExtrAudioTitle, "OK", "OK")
			fDiagEnable()
		EndIf

		$ok = WinExists($g_eacHandle, "")
		If Not $ok Then
			$g_eacHandle = WinGetHandle($g_eacTitle, $g_eacText)
			If @error = 0 Then $ok = 1
		EndIf
	Wend
	
	fDebugPrint("")
	fDebugPrint("- END -")
	fDebugPrint("")	
EndFunc ; MainLoop()


Func fWriteIni()
	FileWrite($g_iniFile, _
		"[Settings]" & @crlf & _
		"Version=" & $g_version & @crlf & _
		"ImageExt=" & $g_imageExt & @crlf & _
		"ImageNaming=" & $g_imageNaming & @crlf & _
		"ImageHotKey=" & $g_imageHotKey & @crlf & _
		"TracksHotKey=" & $g_tracksHotKey & @crlf & _
		"TracksHotVal=" & $g_tracksHotVal & @crlf & _
		"VA=" & $g_VA & @crlf & _
		"CreateAllCuesheets=0" & @crlf & _
		"RunCoverDownloader=" & $g_runCoverDownloader & @crlf & _
		"CoverDownloader=" & $g_coverDownloaderExe & @crlf & _
		"CoverDownloaderXUI=0" & @crlf & _
		"CoverDownloaderLocalPath=" & @crlf & _		
		"EAC=" & $g_eacExe & @crlf & _
		"Tools=" & $g_toolsDir & @crlf & _
		"MinimizeCompressionWindow=" & $g_minCompWindow & @crlf & _
		"ProcessPriority=" & $g_processPriority & @crlf & _
		$g_convKey & "=" & $g_convString & @crlf & _
		"Test=0" & @crlf & _
		@crlf)

	FileWrite($g_iniFile, _
		"[UserTrackFormats]" & @crlf & _
		"Flac=0" & @crlf & _
		"Wavpack=1" & @crlf & _
		"LameMP3=1" & @crlf & _
		"NeroAac=0" & @crlf & _
		"iTunesAac=0" & @crlf & _
		"OggEnc2=0" & @crlf & _
		"Tak=0" & @crlf & _
		@crlf)

	FileWrite($g_iniFile, _
		"[UserOutputNames]" & @crlf & _
		"OutRoot=@mymusic@\EAC" & @crlf & _
		"ImageDir_Flac=@OutRoot@\FLAC-images\$cdartist$" & @crlf & _
		"ImageDir_Wavpack=@OutRoot@\WV-images\$cdartist$" & @crlf & _
		"ImageDir_MP3=@OutRoot@\MP3-images\$cdartist$" & @crlf & _
		"ImageDir_Tak=@OutRoot@\TAK-images\$cdartist$" & @crlf & _
		"TrackDir_Flac=@OutRoot@\FLAC\$cdartist$\[$year$] $album$" & @crlf & _
		"TrackDir_Wavpack=@OutRoot@\WV\$cdartist$\[$year$] $album$" & @crlf & _
		"TrackDir_MP3=@OutRoot@\MP3\$cdartist$\[$year$] $album$" & @crlf & _
		"TrackDir_AAC=@OutRoot@\AAC\$cdartist$\[$year$] $album$" & @crlf & _
		"TrackDir_OGG=@OutRoot@\OGG\$cdartist$\[$year$] $album$" & @crlf & _
		"TrackDir_Tak=@OutRoot@\TAK\$cdartist$\[$year$] $album$" & @crlf & _
		"TrackName_SA=$track$ - $title$" & @crlf & _
		"TrackName_VA=$track$ - $title$ [$artist$]" & @crlf & _
		"TrackName_SA_acdir=$n - $~t" & @crlf & _
		"TrackName_VA_acdir=$n - $~t [$~a]" & @crlf & _
		@crlf)

	FileWrite($g_iniFile, _
		"[UserSettings]" & @crlf & _
		"DebugMode=0" & @crlf & _
		"Debug=0" & @crlf & _
		"Comment=Created with EAC/REACT2, @curdate@" & @crlf & _
		"EmbedCover=1" & @crlf & _
		"ReplayGain=1" & @crlf & _
		"ApplyAlbumGain=1" & @crlf & _
		"AdjustAlbumGain_dB=+3.0" & @crlf & _
		"AddCuesheetAG=0" & @crlf & _
		"UseWaveGainAG=0" & @crlf & _
		"Opt_Flac=-5 -f" & @crlf & _
		"Opt_Wavpack=-hmy" & @crlf & _
		"Opt_LameMP3=-V5 --vbr-new --noreplaygain --nohist" & @crlf & _
		"Opt_NeroAac=-lc -q 0.21" & @crlf & _
		"Opt_iTunesAac=-d -s 2000" & @crlf & _
		"Opt_OggEnc2=-q 3.0" & @crlf & _
		"Opt_Tak=--pN -overwrite" & @crlf & _
		"Ver_Flac=1.1.3" & @crlf & _
		"Ver_Wavpack=4.40" & @crlf & _
		"Ver_LameMP3=3.97" & @crlf & _
		"Ver_NeroAac=1.0.0.2" & @crlf & _
		"Ver_OggEnc2=2.83 Lancer aoTuV b5" & @crlf & _
		"Ver_Tak=1.0.2" & @crlf & _
		@crlf)
EndFunc ; fWriteIni()


Func fHelp()
	Local $URL = "http://wiki.hydrogenaudio.org/index.php?title=REACT"
	RunWait("rundll32.exe url.dll,FileProtocolHandler " & $URL, @WorkingDir)
	;RunWait(@ComSpec & ' /c start ' & $URL, @WorkingDir, @SW_HIDE)
EndFunc

Func fHelpEAC()
	Local $URL = "http://wiki.hydrogenaudio.org/index.php?title=Exact_Audio_Copy"
	RunWait("rundll32.exe url.dll,FileProtocolHandler " & $URL, @WorkingDir)
EndFunc


Func fSetEACTitle()
	WinSetTitle($g_eacTitle, $g_eacText, fGetCoreWinTitle() & "   -   REACT v" & $g_version & " [" & $g_imageExt & "]  Help: F1 / Alt-F1")
EndFunc

Func fCheckEACTitleChange()
	Local $t = fGetCoreWinTitle()
	If $t <> $g_eacFullTitle Then
		fDiagReinitialise()
		$g_eacFullTitle = $t
	EndIf
EndFunc

Func fGetCoreWinTitle()
	Local $t = WinGetTitle($g_eacTitle, $g_eacText)
	Local $p = StringInStr($t, "   -   REACT v", 1)
	If $p > 0 Then $t = StringLeft($t, $p - 1)
	Return $t
EndFunc


Func fReadIni()
	$g_imageExt = IniRead($g_iniFile, "Settings", "ImageExt", $g_imageExt)
	$g_imageNaming	= IniRead($g_iniFile, "Settings", "ImageNaming", "$artist$ - $album$")
	$g_imageHotKey	= IniRead($g_iniFile, "Settings", "ImageHotKey", $g_imageHotKey)
	$g_tracksHotKey	= IniRead($g_iniFile, "Settings", "TracksHotKey", $g_tracksHotKey)
	$g_tracksHotVal	= IniRead($g_iniFile, "Settings", "TracksHotVal", $g_tracksHotVal)
	$g_runCoverDownloader = Int(IniRead($g_iniFile, "Settings", "RunCoverDownloader", $g_runCoverDownloader))
	$g_coverDownloaderExe = IniRead($g_iniFile, "Settings", "CoverDownloader", $g_coverDownloaderExe)
	$g_coverDownloaderXUI = Int(IniRead($g_iniFile, "Settings", "CoverDownloaderXUI", $g_coverDownloaderXUI))
	$g_coverDownloaderLocalPath = IniRead($g_iniFile, "Settings", "CoverDownloaderLocalPath", $g_coverDownloaderLocalPath)
EndFunc


Func fShowIni()
	RunWait('"notepad.exe" "' & $g_iniFile & '"')
	fReadIni()
	fSetEACTitle()
EndFunc

Func fExploreConfig()
	RunWait('"notepad.exe" "' & @ScriptDir & '\' & $g_appName & '-image.cfg"')
EndFunc


Func fConfigEAC()
	Local $mb, $p, $v
	$mb = MsgBox(4+32+8192, $g_appTitle, "Do you want to configure EAC for use with REACT?" & @lf & _
	                                  "If you press No, you can configure EAC later by Ctrl-F2.")
	If $mb = 7 Then Return ; "NO"
	
	fDebugPrint("Configuring EAC for use with REACT...")

	WinClose($g_eacTitle, $g_eacText)
	If WinWaitClose($g_eacTitle, $g_eacText, 5) = 0 Then
		MsgBox(0+48+8192, $g_appTitle, "Cannot close EAC")
		Return
	EndIf

	$p = "HKEY_CURRENT_USER\Software\AWSoftware\EAC\Compression Options"
	RegWrite($p, "UseExternalEncoder", "REG_BINARY", "FF")
	RegWrite($p, "ExternalEncoderType", "REG_BINARY", "14000000")
	RegWrite($p, "ExternalEncoderProgram", "REG_SZ", @ScriptFullPath)
	RegWrite($p, "ExternalEncoderOptions", "REG_SZ", 'REACT %o %s %d "%a" "%g" "%t" "%n" "%x" "%y" "%m" "%e" "%f" "%b" %r')
	RegWrite($p, "ExternalEncoderExtension", "REG_SZ", "eac")
	RegWrite($p, "ExternalEncoderCreateCRC", "REG_BINARY", "00")
	RegWrite($p, "ExternalEncoderID3Tag", "REG_BINARY", "00")
	RegWrite($p, "ExternalEncoderCheckReturnCode", "REG_BINARY", "00")
	RegWrite($p, "ExternalEncoderDeleteSource", "REG_BINARY", "00")

	$p = "HKEY_CURRENT_USER\Software\AWSoftware\EAC\Extraction Options"
	RegWrite($p, "BackgroundExternalCompression", "REG_BINARY", "FF")
	RegWrite($p, "NoCompressionWindow", "REG_BINARY", "00")
	RegWrite($p, "BeepWhenDone", "REG_BINARY", "00")
	RegWrite($p, "AutoSaveStatus", "REG_BINARY", "FF")

	$v = RegRead($p, "FileNamingConvention")
	If $v = "" Or $v = "%T" Then
		RegWrite($p, "UseVariousFileNamingConvention", "REG_BINARY", "FF")
		RegWrite($p, "OpenCDInfoDialog", "REG_BINARY", "FF")
		RegWrite($p, "RetrieveCDDBOnUnknownCD", "REG_BINARY", "FF")
	EndIf

	RegWrite($p, "FileNamingConvention", "REG_SZ", "%D - %C - %N - %T")
	RegWrite($p, "VariousFileNamingConvention", "REG_SZ", "%D - %C - %N - %T (%A)")
	
	; Copy "Use this directory" value to Actual Path
	fSynchOutputDirectories()
	
	MsgBox(0+64+8192, $g_appTitle, "EAC is now configured for use with REACT")
EndFunc


Func fSynchOutputDirectories()
	Local $p, $ds, $ap
	
	; Get value of DirectorySpecification
	$p = "HKEY_CURRENT_USER\Software\AWSoftware\EAC\Extraction Options"
	$ds = RegRead($p, "DirectorySpecification")
	
	; If DirectorySpecification is not set then use the value from OutRoot (or the INI default)
	If StringLen($ds) = 0 Then
		fDebugPrint("DirectorySpecification: Not set")
		$ds = fSubstVars(IniRead($g_iniFile, "UserOutputNames", "OutRoot", "@mymusic@\EAC")) & "\"
		fDebugPrint("Setting DirectorySpecification to OutRoot: " & $ds)
		RegWrite($p, "DirectorySpecification", "REG_SZ", $ds)
	Else
		fDebugPrint("DirectorySpecification: " & $ds)
	EndIf
	
	; Check the value of ActualPath 
	$p = "HKEY_CURRENT_USER\Software\AWSoftware\EAC\StartUp Options"
	$ap = RegRead($p, "ActualPath")
	fDebugPrint("ActualPath: " & $ap)	
	
	; If the paths differ synchronise them (copy DirectorySpecification to ActualPath)
	If $ap <> $ds Then
		RegWrite($p, "ActualPath", "REG_SZ", $ds)
		fDebugPrint("Extraction Options\DirectorySpecification copied to StartUp Options\ActualPath")
	Else
		fDebugPrint("Extraction Options\DirectorySpecification and StartUp Options\ActualPath already in synch")
	EndIf
EndFunc


Func fGUISetVars()
	Local $album
	$gui_album = ControlGetText($g_eacTitle, $g_eacText, "myedit1")
	$gui_artist = ControlGetText($g_eacTitle, $g_eacText, "myedit2")
	$gui_year = ControlGetText($g_eacTitle, $g_eacText, "myedit3")
	$gui_genre = ControlGetText($g_eacTitle, $g_eacText, "mycombo1")
	$gui_freedb = ControlGetText($g_eacTitle, $g_eacText, "mycombo2")
	If $gui_genre = "" And $gui_freedb <> "" Then
		ControlSetText($g_eacTitle, $g_eacText, "mycombo1", $gui_freedb)
		ControlSend($g_eacTitle, $g_eacText, "mycombo1", "{ENTER}")
		$gui_genre = $gui_freedb
	EndIf

	$g_directory = RegRead("HKEY_CURRENT_USER\Software\AWSoftware\EAC\Extraction Options", "DirectorySpecification")
	$album = fSubstIllegalCharsWithSpaces($gui_album)
	If StringRight($g_directory, 1) <> '\' Then $g_directory = $g_directory & '\'
	; The format that EAC uses
	$gui_albumfile = $g_directory & $album
	; The format that the processing instance will use
	$gui_albumfile2 = $g_directory & fSubstIllegalCharsWithSpaces(StringReplace($gui_album, '"', "'"))
	
	$g_infocuesheet = $g_directory & $g_appName & ".cue"
EndFunc


Func fGUISubstVars($string)
	Local $artist_fn = fSubstIllegalChars($gui_artist, 1)
	$string = StringReplace($string, "$album$", fSubstIllegalChars($gui_album, 1))
	$string = StringReplace($string, "$cdartist$", $artist_fn)
	$string = StringReplace($string, "$artist$", $artist_fn)
	$string = StringReplace($string, "$year$", $gui_year)
	$string = StringReplace($string, "$genre$", fSubstIllegalChars($gui_genre))
	$string = StringReplace($string, "$freedb$", $gui_freedb)
	Return $string
EndFunc


Func fSubstIllegalChars($str, $stripTrailingDots = 0)
	Return fSubstIllegalCharsUsing($str, $g_convTable, $stripTrailingDots)
EndFunc


Func fSubstIllegalCharsLikeEac($str, $stripTrailingDots = 0)
	Return fSubstIllegalCharsUsing($str, StringSplit(",|-|-| |!|'|x|[|]", "|"), $stripTrailingDots)
EndFunc


Func fSubstIllegalCharsWithSpaces($str, $stripTrailingDots = 0)
	Return fSubstIllegalCharsUsing($str, StringSplit(" | | | | | | | | ", "|"), $stripTrailingDots)
EndFunc


Func fSubstIllegalCharsUsing($str, $arrCharMap, $stripTrailingDots)
	$str = StringReplace($str, '/', $arrCharMap[1], 0, 1)
	$str = StringReplace($str, '\', $arrCharMap[2], 0, 1)
	$str = StringReplace($str, ':', $arrCharMap[3], 0, 1)
	$str = StringReplace($str, '?', $arrCharMap[4], 0, 1)
	$str = StringReplace($str, '|', $arrCharMap[5], 0, 1)
	$str = StringReplace($str, '"', $arrCharMap[6], 0, 1)
	$str = StringReplace($str, '*', $arrCharMap[7], 0, 1)
	$str = StringReplace($str, '<', $arrCharMap[8], 0, 1)
	$str = StringReplace($str, '>', $arrCharMap[9], 0, 1)
	If $arrCharMap[0] > 9 Then $str = StringReplace($str, ' ', $arrCharMap[10], 0, 1)
	If $stripTrailingDots = 1 Then
		While StringRight($str, 1) == ' ' Or StringRight($str, 1) == '.'
			$str = StringTrimRight($str, 1)
		Wend
	EndIf	
	Return $str
EndFunc


Func fGUIRunCoverDownloader()
	fGUIRunWaitCoverDownloader(0)
EndFunc


Func fGUIRunWaitCoverDownloader($wait)
	fGUISetVars()
	Local $strRealPath, $strLocalPath
	
	; If a local path has been specified
	If $g_coverDownloaderLocalPath <> "" Then
		fDebugPrint("CoverDownloaderLocalPath: " &  $g_coverDownloaderLocalPath)
		
		; Replace tokens with values		
		$strRealPath = $g_coverDownloaderLocalPath		
		$strRealPath = StringReplace($strRealPath, "$artist$", fSubstIllegalChars($gui_artist))
		$strRealPath = StringReplace($strRealPath, "$album$", fSubstIllegalChars($gui_album))
		$strRealPath = StringReplace($strRealPath, "$year$", fSubstIllegalChars($gui_year))	
		
		; Check whether the cover exists locally
		If FileExists($strRealPath) Then
			fDebugPrint("Local cover art file exists")
			
			; Copy the cover to the working directory
			FileCopy($strRealPath, $gui_albumfile2 & '.jpg')

			; Stop there
			Return			
		EndIf
		fDebugPrint("Local cover art file does not exist")
		
		; Create a local path for Album Art XUI
		$strLocalPath = $g_coverDownloaderLocalPath
		$strLocalPath = StringReplace($strLocalPath, "$artist$", '%artist%')
		$strLocalPath = StringReplace($strLocalPath, "$album$", '%album%')
	EndIf
	
	If $g_coverDownloaderXUI Then
		$g_coverDownloaderParams = '/artist "' & StringReplace($gui_artist, '"', '') _
			& '" /album "' & StringReplace($gui_album, '"', '') _
			& '" /path "' & $gui_albumfile2 & '.%extension%" /autoclose'
		If StringLen($strLocalPath) > 0 Then
			$g_coverDownloaderParams = $g_coverDownloaderParams & ' /localImagesPath "' & $strLocalPath & '"'
		EndIf
	Else
		$g_coverDownloaderParams = '"' & StringReplace($gui_artist, '"', '') _
			& '" "' & StringReplace($gui_album, '"', '') & '" "' & $gui_albumfile2 & '.jpg"'
	EndIf
	fDebugPrint("CoverDownloader command: " & '"' & $g_coverDownloaderExe & '" ' & $g_coverDownloaderParams)
	Local $cmd = '"' & $g_coverDownloaderExe & '" ' & $g_coverDownloaderParams
	IF $wait Then
		RunWait($cmd)
	Else
		Run($cmd)
	EndIf
EndFunc


Func fGUICreateGleanCuesheet()
	fDebugPrint("Creating a cuesheet to glean information")
	Local $funcname = "fGUICreateGleanCuesheet"
	Local $orgcuesheet = $gui_albumfile & ".CUE"

	WinWaitActive($g_eacTitle, $g_eacText)
	ControlSend($g_eacTitle, $g_eacText, "", "!s")
	If Not fMoveCuesheet($orgcuesheet, $g_infocuesheet, $funcname) Then Return False
	Return True
EndFunc
	

Func fGUICreateCuesheets()
	fDebugPrint("Creating cuesheets")
	Local $funcname = "fGUICreateCuesheets"
	Local $orgcuesheet = $gui_albumfile & ".CUE"
	
	WinWaitActive($g_eacTitle, $g_eacText)
	ControlSend($g_eacTitle, $g_eacText, "", "!s")
	If Not fMoveCuesheet($orgcuesheet, $gui_albumfile2 & ".[s].cue", $funcname) Then Return False
	WinWaitActive($g_eacTitle, $g_eacText)
	WinMenuSelectItem($g_eacTitle, $g_eacText, $g_cueMenu[1], $g_cueMenu[2], $g_cueMenu[3])
	If Not fMoveCuesheet($orgcuesheet, $gui_albumfile2 & ".[mg].cue", $funcname) Then Return False
	WinWaitActive($g_eacTitle, $g_eacText)
	ControlSend($g_eacTitle, $g_eacText, "", "!m")
	If Not fMoveCuesheet($orgcuesheet, $gui_albumfile2 & ".[m].cue", $funcname) Then Return False
	WinWaitActive($g_eacTitle, $g_eacText)
	ControlSend($g_eacTitle, $g_eacText, "", "!o")
	If Not fMoveCuesheet($orgcuesheet, $gui_albumfile2 & ".[mc].cue", $funcname) Then Return False
	Return True
EndFunc


Func fCreateCuesheetsTest()
	If Int(IniRead($g_iniFile, "Settings", "CreateAllCuesheets", 0)) And _
	   RegRead("HKEY_CURRENT_USER\Software\AWSoftware\EAC\Extraction Options", "DirectoryUse") = "00000000" Then
		Return 1
	EndIf
	Return 0
EndFunc


Func fMoveCuesheet($src, $dest, $returnfunc)
	Local $i = 0
	fDebugPrint("Moving '" & $src & "' to '" & $dest & "'")

	; Wait for up to 5 seconds, or until the Gap Detection 'Analyzing' window closes (whichever is greater)
	While ($i <= 20) Or WinExists("Analyzing", "Detecting Pre-Track Gaps")
		If FileExists($src) Then ExitLoop
		Sleep(250)
		$i = $i + 1
	Wend
	
	; Wait for up to 5 seconds for the cuefile to appear.
	; EAC is sometimes slow to create the actual cuefile after "Analyzing" window is closed.
	; This reduces missing file warning messages to users.
	For $i = 1 To 20
		If FileExists($src) Then ExitLoop
		fDebugPrint("Waiting for the cuefile to appear")
		Sleep(250)
	Next
	
	If FileExists($src) Then
		FileMove($src, $dest, 1)
	Else
		fDebugPrint("Unable to move '" & $src & "'. File does not exist")
		Local $r = MsgBox(2+48+8192, "REACT: Missing Cuesheet", "Unable to move cuesheet. File does not exist.")
		If $r = 4 Then
			fDebugPrint("Retrying...")
			Return Call($returnfunc)
		ElseIf $r = 3 Then
			fDebugPrint("Aborting...")
			; Delete all cuefiles created so far.. if any.
			FileDelete($g_directory & "*.cue")
			fDebugPrint("All cuefiles deleted from '" & $g_directory & "' directory")
			fDebugPrint("")
			fDebugPrint("- END -")
			fDebugPrint("")
			Return False
		Else
			fDebugPrint("Ignoring...")
		EndIf
	EndIf
	
	Return True
EndFunc


Func fTracksCopyCompressed()
	fGUISetVars()
	fDiagDisable()
	fWriteMetaDataToIni()

	FileDelete($gui_albumfile & '.log')
	FileDelete($g_directory & 'processing[*].tmp')
	
	If Not fGUICreateGleanCuesheet() Then
		fDiagEnable()
		Return
	EndIf
	
	If fCreateCuesheetsTest() Then
		If Not fGUICreateCuesheets() Then
			fDiagEnable()
			Return
		EndIf
		
		; Add additional meta data to all track cuesheets
		fLoadMetaDataFromIni()
		fLoadAndResaveCuesheet($gui_albumfile2 & ".[s].cue")
		fLoadAndResaveCuesheet($gui_albumfile2 & ".[mg].cue")
		fLoadAndResaveCuesheet($gui_albumfile2 & ".[m].cue")
		fLoadAndResaveCuesheet($gui_albumfile2 & ".[mc].cue")
	EndIf
	
	If $g_runCoverDownloader Then fGUIRunWaitCoverDownloader($g_runCoverDownloader < 3)

	Local $trackMenu
	If Int(IniRead($g_iniFile, "Settings", "Test", 0)) Or $g_tracksHotVal = '+{F6}' Then
		$trackMenu = StringSplit("&Action|Test && C&opy Selected Tracks|Compressed...", "|")
		fDebugPrint("Test & Copy Tracks Beginning...")
	Else
		$trackMenu = StringSplit("&Action|&Copy Selected Tracks|Compressed...", "|")
		fDebugPrint("Copy Tracks Beginning...")
	EndIf
	WinMenuSelectItem($g_eacTitle, $g_eacText, $trackMenu[1], $trackMenu[2], $trackMenu[3])

	fDebugPrint("")
	fDebugPrint("- END -")
	fDebugPrint("")
EndFunc


Func fImageCopyCompressed()
	fGUISetVars()
	fDiagDisable()
	fWriteMetaDataToIni()

	FileDelete($gui_albumfile & '.log')
	
	If fCreateCuesheetsTest() Then
		If Not fGUICreateCuesheets() Then
			fDiagEnable()
			Return
		EndIf
		
		; Add additional meta data to all track cuesheets
		fLoadMetaDataFromIni()
		fLoadAndResaveCuesheet($gui_albumfile2 & ".[s].cue")
		fLoadAndResaveCuesheet($gui_albumfile2 & ".[mg].cue")
		fLoadAndResaveCuesheet($gui_albumfile2 & ".[m].cue")
		fLoadAndResaveCuesheet($gui_albumfile2 & ".[mc].cue")
	EndIf
	
	If $g_runCoverDownloader Then fGUIRunWaitCoverDownloader(BitAnd($g_runCoverDownloader, 2))

	Local $imgMenu
	If Int(IniRead($g_iniFile, "Settings", "Test", 0)) Then
		$imgMenu = StringSplit("&Action|Test && Copy Image && Create CUE Sheet|Compressed...", "|")
		fDebugPrint("Test & Copy Image Beginning...")
	Else
		$imgMenu = StringSplit("&Action|Copy &Image && Create CUE Sheet|Compressed...", "|")
		fDebugPrint("Copy Image Beginning...")
	EndIf

	Local $name = fGUISubstVars($g_imageNaming)
	WinMenuSelectItem($g_eacTitle, $g_eacText, $imgMenu[1], $imgMenu[2], $imgMenu[3])
	WinWait($g_saveWaveformStr, "", 5)
	ControlSetText($g_saveWaveformStr, "", "Edit1", $name)
	; Do it again after 0.2s in case it failed...
	Sleep(250)
	ControlSetText($g_saveWaveformStr, "", "Edit1", $name)

	If RegRead("HKEY_CURRENT_USER\Software\AWSoftware\EAC\Extraction Options", "DirectoryUse") = "00000000" Then
		Sleep(250)
		ControlSend($g_saveWaveformStr, "", "Edit1", "{ENTER}")
	EndIf
	
	fDebugPrint("")
	fDebugPrint("- END -")
	fDebugPrint("")	
EndFunc



#cs  EAC
  1  <config>
  2  %o        original filename (no dir, and without temporary renaming)
  3  %s        temporary source filename
  4  %d        temporary destination filename
  5  %a        @artist@
  6  %g        @album@
  7  %t        @title@
  8  %n        @track@
  9  %x        @numtracks@
  10 %y        @year@
  11 %m        @genre@
  12 %e        @comment@
  13 %f        @freedbid@
  14 %b        @crc@
  15 %r        @bitrate@
#ce

Func fEncode()
	Local $a, $val, $temp_root, $temp_ext, $hasCue, $cuesheet_org
	Local $batdata, $pid, $winState
	Local $cuedata_org, $cuedata, $configFile, $basepath
	Local $cuesheetExt = "cue"
	Local $old_eaclog
	
	; Are we in debug mode?
	$g_debugging = Int(IniRead($g_iniFile, "UserSettings", "DebugMode", "0"))

	; Determine temp compression extension - normally .eac
	$a = StringSplit($CmdLine[4], ".")
	$temp_ext = $a[$a[0]]
	$temp_root = StringTrimRight($CmdLine[4], StringLen($temp_ext) + 1) & "-react"

	; Determine source/destination directory
	$a = StringSplit($CmdLine[4], "\")
	$g_sourcedir = StringTrimRight($CmdLine[4], StringLen($a[$a[0]]) + 1) ; remove tmp name
	$g_basename = StringTrimRight($CmdLine[2], 4) ; remove ".wav" ext.
	$basepath = $g_sourcedir & "\" & $g_basename

	; The real source .wav
	$g_source_tmp = $CmdLine[3]
	$g_artist = StringReplace($CmdLine[5], '"', "'", 0, 1)
	$g_album = StringReplace($CmdLine[6], '"', "'", 0, 1)
	$g_title = StringReplace($CmdLine[7], '"', "'", 0, 1)
	$g_track = Int($CmdLine[8]) ; force to number
	$g_numtracks = Int($CmdLine[9])
	$g_year = $CmdLine[10]
	$g_genre = $CmdLine[11]
	$g_comment = StringReplace($CmdLine[12], '"', "'", 0, 1)
	$g_freedbID = $CmdLine[13]
	$g_crc = $CmdLine[14]
	$g_bitrate = $CmdLine[15]

	; This block is a workaround for a bug in parameter parsing in AutoIt3.1.1
	If $g_artist = "'" Then $g_artist = ""
	If $g_album = "'" Then $g_album = ""
	If $g_title = "'" Then $g_title = ""
	If $g_year = '"' Then $g_year = ""
	If $g_genre = '"' Then $g_genre = ""
	If $g_comment = "'" Then $g_comment = ""

	$g_artist_fn = fSubstIllegalChars($g_artist, 1)
	$g_album_fn = fSubstIllegalChars($g_album, 1)
	$g_title_fn = fSubstIllegalChars($g_title)
	$g_track_fn = StringFormat("%02d", $g_track) ; force to number
	$g_numtracks_fn = StringFormat("%02d", $g_numtracks) ; force to number
	$g_genre_fn = fSubstIllegalChars($g_genre)
	$g_comment_fn = fSubstIllegalChars($g_comment)

	$g_VA = IniRead($g_iniFile, "Settings", "VA", $g_VA) ; Various Artists

	If StringLeft($CmdLine[2], 7) = "Various" Then
		$g_cdartist = $g_VA
		$g_cdartist_fn = $g_VA
		$g_various = 1
	Else
		$g_cdartist = $g_artist
		$g_cdartist_fn = $g_artist_fn
	EndIf
	
	If $g_various Then
		fDebugPrint("Various artist: Yes")
	Else
		fDebugPrint("Various artist: No")
	EndIf
	
	; Load additional meta data from ini file into $g_arrMetaData
	fLoadMetaDataFromIni()

	$g_toolsDir = FileGetShortName(IniRead($g_iniFile, "Settings", "Tools", $g_toolsDir))
	$g_processPriority = IniRead($g_iniFile, "Settings", "ProcessPriority", $g_processPriority)
	$g_userTrackFormats = IniReadSection($g_iniFile, "UserTrackFormats")
	$g_userOutputNames = IniReadSection($g_iniFile, "UserOutputNames")
	$g_userSettings = IniReadSection($g_iniFile, "UserSettings")

	; Get image extension from ini-file
	$g_imageExt = IniRead($g_iniFile, "Settings", "ImageExt", $g_imageExt)

	; Check if a cuesheet was created ==> cd image.

	$cuesheet_org = $basepath & ".cue"
	$hasCue = FileExists($cuesheet_org)
	If Not $hasCue Then
		$cuesheet_org = $basepath & "." & $temp_ext & ".cue"
		$hasCue = FileExists($cuesheet_org)
	EndIf

	If $hasCue Then
		fDebugPrint("Rip mode: Image")
	Else
		fDebugPrint("Rip mode: Tracks")
	EndIf

	; Log and cover art basename

	If $hasCue Then
		$g_logfile = $g_basename
	Else
		If $g_various Then
			$g_logfile = StringLeft($g_basename, StringInStr($g_basename, "-") + 1) & fSubstIllegalCharsWithSpaces($g_album)
		Else
			$g_logfile = fSubstIllegalCharsWithSpaces($g_cdartist & " - " & $g_album)
		EndIf
	EndIf
	
	$g_eaclog = $g_sourcedir & "\" & $g_logfile & ".log"
	$g_albumfile = $g_sourcedir & "\" & fSubstIllegalCharsWithSpaces($g_album)
	
	; Get real total tracks
	If Not $hasCue Then
		$g_infocuesheet = $g_sourcedir & "\" & $g_appName & ".cue"
		fGetInfoFromCuesheet($g_infocuesheet)
	EndIf
	
	If $hasCue Then
		$configFile = @ScriptDir & '\' & $g_appName & '-image.cfg'
	Else
		$configFile = @ScriptDir & '\' & $g_appName & '-track.cfg'
	EndIf

	If Not FileExists($configFile) Then
		MsgBox(0+48+8192, $g_appTitle, "No configuration file found: " & $configFile, 5)
		Return
	EndIf
	
	fDebugPrint("Command line: " & $CmdLineRaw)
	fDebugPrint("Source directory: " & $g_sourcedir)
	fDebugPrint("Base name: " & $g_basename)
	fDebugPrint("Artist: " & $g_artist)
	fDebugPrint("Album: " & $g_album)
	fDebugPrint("Title: " & $g_title)
	fDebugPrint("Track: " & $g_track)
	fDebugPrint("Total tracks: " & $g_numtracks)
	fDebugPrint("Year: " & $g_year)
	fDebugPrint("Genre: " & $g_genre)
	fDebugPrint("Comment: " & $g_comment)
	fDebugPrint("FreeDB ID: " & $g_freedbID)
	fDebugPrint("CRC: " & $g_crc)
	fDebugPrint("Bitrate: " & $g_bitrate)	

	If $hasCue Then
		$cuedata_org = FileRead($cuesheet_org, FileGetSize($cuesheet_org))
		If @error = -1 Then
			fDebugPrint("Could not open cuesheet '" & $cuesheet_org & "'")
		EndIf
		
		; Add Additional Meta Data to the cuesheet as REM statements
		$cuedata_org = fAddAdditionalMetaDataToCuesheet($cuedata_org)

		; --------- Set the temp sourcecuesheet -----------
		Local $c1 = StringSplit($cuedata_org, 'FILE "', 1)
		Local $c2 = StringSplit($cuedata_org, '" WAVE', 1)
		$a = StringSplit($g_source_tmp, "\")
		$cuedata = $c1[1] & 'FILE "' & $a[$a[0]] & '" WAVE' & $c2[2]

		$g_sourcecuesheet = $temp_root & ".cue"
		FileDelete($g_sourcecuesheet)
		FileWrite($g_sourcecuesheet, $cuedata)

		; --------- Set the destination cuesheet -----------
		Local $cueDataType = 'WAVE'
		If $g_imageExt = "mp3" Then $cueDataType = 'MP3'

		$g_destimage = $basepath & "." & $g_imageExt
		$g_cuesheet = $basepath & "." & $cuesheetExt

		$cuedata = StringReplace($cuedata_org, '.' & $temp_ext & '" WAVE', '.' & $g_imageExt & '" ' & $cueDataType)
	EndIf

	; Determine windows state to set
	If RegRead("HKEY_CURRENT_USER\Software\AWSoftware\EAC\Extraction Options", "NoCompressionWindow") = "FF" Then
		$winState = @SW_HIDE
	ElseIf Int(IniRead($g_iniFile, "Settings", "MinimizeCompressionWindow", $g_minCompWindow)) Then
		$winState = @SW_MINIMIZE
	Else
		$winState = @SW_SHOW
	EndIf

	; Create the final cuesheet.
	If $hasCue Then
		FileDelete($cuesheet_org)
		FileDelete($g_cuesheet)
		; Save the new .cue file
		FileWrite($g_cuesheet, $cuedata)
	EndIf
	
	$g_batfile = $g_sourcedir & "\" & $g_logfile & "[" & $g_track & "].bat"

	; Read the config file
	Local $cmd = FileRead($configFile, FileGetSize($configFile))
	
	; Parse TrackName value
	If $g_various Then
		$g_trackName = IniRead($g_iniFile, "UserOutputNames", "TrackName_VA", "$track$ - $title$ [$artist$]")
	Else
		$g_trackName = IniRead($g_iniFile, "UserOutputNames", "TrackName_SA", "$track$ - $title$")
	EndIf
	$g_trackName = fStringReplaceEscape(fSubstVars($g_trackName))
	
	; Make sure the EAC log file is written before continuing - we want to embed it in image
	If $hasCue Or $g_track = $g_numtracks Then
	
		; Check for normal log file name (0.99 format and no double quote/apostrophe issue)
		For $i=1 To 12 ; max 3 seconds
			If FileExists($g_eaclog) Then
				$g_logfile = $g_sourcedir & "\" & $g_logfile
				ExitLoop
			EndIf
			Sleep(250)
		Next

		; If the log file hasn't been found we need to get inventive
		If Not FileExists($g_eaclog) Then
			Local $objSearch, $strLog
			$objSearch = FileFindFirstFile($g_sourcedir & "\*.log")  
			$i = 0
			While ($objSearch = -1) And ($i < 10)
				Sleep(500)
				$i = $i + 1
			Wend
			While 1
				$strLog = FileFindNextFile($objSearch) 
				If @error Then ExitLoop
				If fFileIsLogfile($strLog) Then
					$g_eaclog = $g_sourcedir & "\" & $strLog
					$g_logfile = $g_sourcedir & "\" & StringLeft($strLog, StringLen($strLog) - 4)
					ExitLoop
				EndIf
			Wend
			FileClose($objSearch)
		EndIf
		
		FileDelete($g_infocuesheet)
	Else
		Sleep(500)
	EndIf
	
	;Check for album art, default to jpeg
	If FileExists($g_albumfile & ".gif") Then
		$g_cover = $g_albumfile & ".gif"
	ElseIf FileExists($g_albumfile & ".bmp") Then
		$g_cover = $g_albumfile & ".bmp"		
	ElseIf FileExists($g_albumfile & ".png") Then
		$g_cover = $g_albumfile & ".png"
	Else
		$g_cover = $g_albumfile & ".jpg"
	EndIf
	
	fDebugPrint("Log: " & $g_eaclog)
	fDebugPrint("Album file: " & $g_albumfile)
	fDebugPrint("Log file: " & $g_logfile)
	fDebugPrint("Cover file: " & $g_cover)
	
	; Create the .bat file
	$batdata = fSubstVars($cmd)
	
	FileDelete($g_batfile)
	FileWrite($g_batfile, $batdata)
	
	; Finally, execute the .bat file
	$pid = Run('"' & $g_batfile & '"', $g_sourcedir, $winState)
	ProcessSetPriority($pid, $g_processPriority)
	ProcessWaitClose($pid)

	; Remove temporay files
	FileDelete($g_sourcecuesheet)

	Sleep(200)
	
	fDebugPrint("")
	fDebugPrint("- END -")
	fDebugPrint("")
EndFunc


Func fSubstVars($str)
	For $i = $g_userSettings[0][0] To 1 Step -1
		$str = fStringReplaceForDOS($str, '@' & $g_userSettings[$i][0] & '@', $g_userSettings[$i][1])
	Next
	For $i = $g_userOutputNames[0][0] To 1 Step -1
		$str = StringReplace($str, '@' & $g_userOutputNames[$i][0] & '@', $g_userOutputNames[$i][1])
	Next
	For $i = $g_userTrackFormats[0][0] To 1 Step -1
		$str = StringReplace($str, '@' & $g_userTrackFormats[$i][0] & '@', $g_userTrackFormats[$i][1])
	Next
	$str = StringReplace($str, "@trackname@", $g_trackName)
	
	$str = fStringReplaceForDOS($str, "$cdartist$", $g_cdartist_fn)
	$str = fStringReplaceForDOS($str, "$artist$", $g_artist_fn)
	$str = fStringReplaceForDOS($str, "$album$", $g_album_fn)
	$str = fStringReplaceForDOS($str, "$track$", $g_track_fn)
	$str = fStringReplaceForDOS($str, "$numtracks$", $g_numtracks_fn)
	$str = fStringReplaceForDOS($str, "$title$", $g_title_fn)
	$str = fStringReplaceForDOS($str, "$year$", $g_year)
	$str = fStringReplaceForDOS($str, "$genre$", $g_genre_fn)
	$str = fStringReplaceForDOS($str, "$comment$", $g_comment_fn)

	$str = fStringReplaceForDOS($str, "@cdartist@", $g_cdartist)
	$str = fStringReplaceForDOS($str, "@artist@", $g_artist)
	$str = fStringReplaceForDOS($str, "@album@", $g_album)
	$str = fStringReplaceForDOS($str, "@title@", $g_title)
	$str = fStringReplaceForDOS($str, "@track@", $g_track)
	$str = fStringReplaceForDOS($str, "@numtracks@", $g_numtracks)
	$str = fStringReplaceForDOS($str, "@year@", $g_year)
	$str = fStringReplaceForDOS($str, "@genre@", $g_genre)
	$str = fStringReplaceForDOS($str, "@freedbid@", $g_freedbID)
	$str = fStringReplaceForDOS($str, "@comment@", $g_comment)

	$str = fStringReplaceForDOS($str, "@source@", $g_source_tmp)
	$str = fStringReplaceForDOS($str, "@image@", $g_destimage)
	$str = fStringReplaceForDOS($str, "@imageext@", $g_imageExt)
	$str = fStringReplaceForDOS($str, "@cuesheet@", $g_cuesheet)
	$str = fStringReplaceForDOS($str, "@sourcecuesheet@", $g_sourcecuesheet)
	$str = fStringReplaceForDOS($str, "@various@", $g_various)
	$str = fStringReplaceForDOS($str, "@va@", $g_VA)

	$str = fStringReplaceForDOS($str, "@crc@", $g_crc)
	$str = fStringReplaceForDOS($str, "@bitrate@", $g_bitrate)
	$str = fStringReplaceForDOS($str, "@sourcedir@", $g_sourcedir)
	$str = fStringReplaceForDOS($str, "@basename@", $g_basename)
	$str = fStringReplaceForDOS($str, "@albumfile@", $g_albumfile)
	$str = fStringReplaceForDOS($str, "@logfile@", $g_logfile)
	$str = fStringReplaceForDOS($str, "@cover@", $g_cover)
	$str = fStringReplaceForDOS($str, "@eaclog@", $g_eaclog)
	$str = fStringReplaceForDOS($str, "@tools@", $g_toolsDir)
	$str = fStringReplaceForDOS($str, "@reactdir@", @ScriptDir)
	$str = fStringReplaceForDOS($str, "@mymusic@", $g_myMusicDir)
	$str = fStringReplaceForDOS($str, "@curdate@", @YEAR &'-'& @MON &'-'& @MDAY)
	
	; Replace additional meta data tokens
	$str = fReplaceAdditionalMetaData($str, $g_track)

	Return $str
EndFunc

Func fStringReplaceForDOS($strString, $strOld, $strNew)
	Return StringReplace($strString, $strOld, StringReplace($strNew, "%", "%%"))
EndFunc

Func fStringReplaceEscape($strString)
	$strString = StringReplace($strString, "^", "^^")
	$strString = StringReplace($strString, "&", "^&")
	Return $strString
EndFunc

Func fDebugPrint($strText)
	Local $strTimestamp = ""
	If $g_debugging Then
		If $strText <> "" Then $strTimestamp = @YEAR & "-" & @MON & "-"  & @MDAY & " "  & @HOUR & ":"  & @MIN & ":"  & @SEC & "  "
		FileWrite(@ScriptDir & "\debugging.txt" , $strTimestamp & $strText & @CRLF)
	EndIf
EndFunc

Func fFileIsLogfile($strFile)
	Local $strNewPattern = StringReplace(fPregQuote($g_logfile), "'", "[ ']")
	Local $strOldPattern = StringReplace(fPregQuote(fSubstIllegalCharsWithSpaces($g_album)), "'", "[ ']")
	If StringRegExp($strFile, "\A" & $strNewPattern & "\.log\z") Then
		Return True
	ElseIf StringRegExp($strFile, "\A" & $strOldPattern & "\.log\z") Then
		Return True
	EndIf
	Return False
EndFunc

Func fPregQuote($strText)
	; PHP's preg_quote chars: \ + * ? [ ^ ] $ ( ) { } = ! < > | : 
	$strText = StringReplace($strText, '\', '\\')
	$strText = StringReplace($strText, '+', '\:')
	$strText = StringReplace($strText, '*', '\*')
	$strText = StringReplace($strText, '?', '\?')
	$strText = StringReplace($strText, '[', '\[')
	$strText = StringReplace($strText, '^', '\^')
	$strText = StringReplace($strText, ']', '\]')
	$strText = StringReplace($strText, '$', '\$')
	$strText = StringReplace($strText, '(', '\(')
	$strText = StringReplace($strText, ')', '\)')
	$strText = StringReplace($strText, '{', '\{')
	$strText = StringReplace($strText, '}', '\}')
	$strText = StringReplace($strText, '=', '\=')
	$strText = StringReplace($strText, '!', '\!')
	$strText = StringReplace($strText, '<', '\<')
	$strText = StringReplace($strText, '>', '\>')
	$strText = StringReplace($strText, '|', '\|')
	$strText = StringReplace($strText, ':', '\:')
	Return $strText
EndFunc

Func fGetInfoFromCuesheet($strFilename)
	Local $i, $arrMatch
	Local $strCuesheet = FileRead($strFilename)
	$arrMatch = StringRegExp($strCuesheet, "TRACK (\d+) AUDIO", 3)
	If @error == 0 Then
		$g_numtracks = Int($arrMatch[UBound($arrMatch) - 1])
		$g_numtracks_fn = StringFormat("%02d", $g_numtracks) ; force to number
	EndIf
EndFunc
