;
; REACT - Run EAC Tasks
; AutoIt3.1.1 script - http://www.autoitscript.com
;
; ----------------------------------------------------------------------------------------------------
; Copyright (C) Tycho, 2006
;
; REACT is provided 'as-is', without any express or implied warranty.
; In no event will the authors be held liable for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any purpose,
; including commercial applications, and to alter it and redistribute it freely,
; subject to the following restrictions (known as zlib license):
;
;   1. The origin of this software must not be misrepresented; you must not claim that you wrote
;      the original software. If you use this software in a product, an acknowledgment in the product
;      documentation would be appreciated but is not required.
;   2. Altered source versions must be plainly marked as such, and must not be misrepresented as being
;      the original software.
;   3. This notice may not be removed or altered from any source distribution.
; ----------------------------------------------------------------------------------------------------
;
; REACT Documentation - http://www.hydrogenaudio.org/forums/index.php?showtopic=35158
; EAC                 - http://www.exactaudiocopy.org

#NoTrayIcon

Opt("MustDeclareVars", 1)
Opt("RunErrorsFatal", 0)


Global $g_version		= "2.0"
Global $g_imageExt		= "wav"
Global $g_appName		= StringTrimRight(@ScriptName, 4)
Global $g_appTitle		= $g_appName & " v" & $g_version
Global $g_iniFile		= @ScriptDir & "\" & $g_appName & ".ini"
Global $g_minCompWindow		= 1
Global $g_processPriority	= 1

Global $g_eacExe		= RegRead("HKEY_CURRENT_USER\Software\AWSoftware\EAC", "InstallPath") & "EAC.exe"
Global $g_myMusicDir		= RegRead("HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders", "My Music")
Global $g_toolsDir		= @ScriptDir & "\tools"
Global $g_coverDownloaderExe	= @ScriptDir & "\coverdownloader\albumart.exe"
Global $g_runCoverDownloader	= 1
Global $g_cover			= ""

Global $g_eacTitle		= "Exact Audio Copy"
Global $g_eacText		= "CD Artist"
Global $g_eacHandle		= 0
Global $g_eacExtrAudioTitle	= "Extracting Audio Data"
Global $g_eacExtrDoneMinTitle	= "[100.0%]    Extracting"
Global $g_imageHotKey		= "{F10}"
Global $g_tracksHotKey		= "{F4}"
Global $g_tracksHotVal		= "+{F5}"
Global $g_VA			= "Various Artists"

Global $g_imgMenu		= StringSplit("&Action|Copy &Image && Create CUE Sheet|Compressed...", "|")
Global $g_saveWaveformStr	= "Save Waveform"
Global $g_imageNaming		= "$artist$ - [$year$] $album$"
Global $g_convKey		= "Sla_Bks_Col_Qst_Bar_Quo_Ast_Lt_Gt"
Global $g_convString		= "-|-|-||!|'|#|[|]"
Global $g_convTable

Global $g_source_tmp
Global $g_destimage
Global $g_cuesheet
Global $g_sourcecuesheet
Global $g_albumfile		= "{undefined}"
Global $g_batfile
Global $g_eaclog
Global $g_sourcedir
Global $g_basename
Global $g_various = 0

Global $g_cueMenu		= StringSplit("&Action|Create CUE &Sheet|Multiple WAV Files With Gaps...  (Noncompliant)", "|")

Global $g_cdartist, $g_artist, $g_album, $g_title, $g_genre, $g_freedb, $g_comment
Global $g_cdartist_fn, $g_artist_fn, $g_album_fn, $g_track_fn, $g_numtracks_fn, $g_title_fn, $g_genre_fn, $g_freedb_fn, $g_comment_fn
Global $g_track, $g_numtracks, $g_year, $g_freedbID, $g_crc, $g_bitrate

Global $gui_album, $gui_artist, $gui_year, $gui_genre, $gui_freedb, $gui_albumfile
Global $g_userTrackFormats = 0, $g_userOutputNames = 0, $g_userSettings = 0


fMain()
Exit


Func fMain()
	Local $appid = "REACT-847323"

	$g_eacExe = IniRead($g_iniFile, "Settings", "EAC", $g_eacExe)
	$g_convString = IniRead($g_iniFile, "Settings", $g_convKey, $g_convString)
	$g_convTable = StringSplit($g_convString, "|")

	If $CmdLine[0] > 0 Then
		fEncode()
	ElseIf Not WinExists($appid) Then
		Opt("TrayIconHide", 0) ; Show tray icon
		AutoItWinSetTitle($appid)
		fMainLoop()
	Else
		Local $pid = Run($g_eacExe)
		If @error = 1 Then
			MsgBox(0+48+8192, $g_appTitle, "Couldn't start " & $g_eacExe)
			Exit
		EndIf
		WinWait($g_eacTitle, $g_eacText)
	EndIf
EndFunc


Func fMainLoop()
	; Handle old ini file.
	If FileExists($g_iniFile) Then
		Local $ver = IniRead($g_iniFile, "Settings", "Version", 0.0)
		If $ver <> $g_version Then
			fReadIni()
			;FileMove($g_iniFile, $g_iniFile & "." & $ver, 1)
			FileDelete($g_iniFile)
			fWriteIni()
			; No need to reconfigure EAC - no changes.
		EndIf
	EndIf

	If Not FileExists($g_iniFile) Then
		fWriteIni()
		fConfigEAC()
	EndIf

	; Check naming scheme
	Local $p = "HKEY_CURRENT_USER\Software\AWSoftware\EAC\Extraction Options"
	Local $e = "FileNamingConvention"
	If RegRead($p, "UseVariousFileNamingConvention" ) = "FF" Then $e = "VariousFileNamingConvention"
	Local $c = RegRead($p, $e)
	If StringLeft($c, 2) <> "%D" Or StringInStr($c, "\") > 0 Then
		MsgBox(0+48+8192, $g_appTitle, "EAC active naming scheme must start with '%D' and not contain '\'" & @crlf & _
					       "Please, press Ctrl-F2 to configure EAC for use with REACT")
	EndIf

	$c = RegRead("HKEY_CURRENT_USER\Software\AWSoftware\EAC\StartUp Options", "UseLanguage" )
	If @error = 0 And $c <> "47AB3DF2" Then
		MsgBox(0+48+8192, $g_appTitle, "EAC must use English language for REACT to work." & @crlf & _
					       "Please, change language settings to English")
	EndIf

	; ---------------- run EAC ---------------------

	Opt("WinTitleMatchMode", 2)

	If Not (WinExists($g_eacTitle, $g_eacText) Or WinExists("]    Extracting") Or WinExists("]    Detecting Gaps")) Then
		Local $pid = Run($g_eacExe)
		If @error = 1 Then
			MsgBox(0+48+8192, $g_appTitle, "Couldn't start " & $g_eacExe)
			Exit
		EndIf
		WinWait($g_eacTitle, $g_eacText)
	EndIf

	$g_eacHandle = WinGetHandle($g_eacTitle, $g_eacText)

	fReadIni()

	HotKeySet("!{F10}", "fGUIRunCoverDownloader")

	Local $hot = 0
	Local $ok = 1

	While $ok
		If WinActive($g_eacTitle, $g_eacText) Then
			If Not $hot Then
				$hot = 1
				HotKeySet($g_imageHotKey, "fImageCopyCompressed")
				HotKeySet($g_tracksHotKey, "fTracksCopyCompressed")
				HotKeySet("{F1}", "fHelpEAC")
				HotKeySet("!{F1}", "fHelp")
				HotKeySet("!{F2}", "fShowIni")
				HotKeySet("^{F2}", "fConfigEAC")
				HotKeySet("!{F3}", "fExploreConfig")
				fSetEACTitle()
			EndIf
		ElseIf $hot Then
			$hot = 0
			HotKeySet($g_imageHotKey)
			HotKeySet($g_tracksHotKey)
			HotKeySet("{F1}")
			HotKeySet("!{F1}")
			HotKeySet("!{F2}")
			HotKeySet("^{F2}")
			HotKeySet("!{F3}")
		EndIf

		If WinExists($g_eacExtrDoneMinTitle) Then
			; Restore if minimized
			If BitAnd(WinGetState($g_eacExtrDoneMinTitle), 16) <> 0 Then WinSetState($g_eacExtrDoneMinTitle, "", @SW_RESTORE)
		EndIf

		Sleep(250)

		If WinExists($g_eacExtrAudioTitle, "OK") And Not WinExists($g_eacExtrAudioTitle, "Cancel") Then
			ControlClick($g_eacExtrAudioTitle, "OK", "OK")
		EndIf

		$ok = WinExists($g_eacHandle, "")
		If Not $ok Then
			$g_eacHandle = WinGetHandle($g_eacTitle, $g_eacText)
			If @error = 0 Then $ok = 1
		EndIf
	Wend
EndFunc ; MainLoop()


Func fWriteIni()
	FileWrite($g_iniFile, _
		"[Settings]" & @crlf & _
		"Version=" & $g_version & @crlf & _
		"ImageExt=" & $g_imageExt & @crlf & _
		"ImageNaming=" & $g_imageNaming & @crlf & _
		"ImageHotKey=" & $g_imageHotKey & @crlf & _
		"TracksHotKey=" & $g_tracksHotKey & @crlf & _
		"TracksHotVal=" & $g_tracksHotVal & @crlf & _
		"VA=" & $g_VA & @crlf & _
		"CreateAllCuesheets=0" & @crlf & _
		"RunCoverDownloader=" & $g_runCoverDownloader & @crlf & _
		"CoverDownloader=" & $g_coverDownloaderExe & @crlf & _
		"EAC=" & $g_eacExe & @crlf & _
		"Tools=" & $g_toolsDir & @crlf & _
		"MinimizeCompressionWindow=" & $g_minCompWindow & @crlf & _
		"ProcessPriority=" & $g_processPriority & @crlf & _
		$g_convKey & "=" & $g_convString & @crlf & @crlf)

	FileWrite($g_iniFile, _
		"[UserTrackFormats]" & @crlf & _
		"Flac=0" & @crlf & _
		"Wavpack=1" & @crlf & _
		"LameMP3=1" & @crlf & _
		"NeroAac=0" & @crlf & _
		"iTunesAac=0" & @crlf & _
		"OggEnc2=0" & @crlf & @crlf)

	FileWrite($g_iniFile, _
		"[UserOutputNames]" & @crlf & _
		"OutRoot=@mymusic@\EAC" & @crlf & _
		"ImageDir_Flac=@OutRoot@\FLAC-images\$cdartist$" & @crlf & _
		"ImageDir_Wavpack=@OutRoot@\WV-images\$cdartist$" & @crlf & _
		"ImageDir_MP3=@OutRoot@\MP3-images\$cdartist$" & @crlf & _
		"TrackDir_Flac=@OutRoot@\FLAC\$cdartist$\[$year$] $album$" & @crlf & _
		"TrackDir_Wavpack=@OutRoot@\WV\$cdartist$\[$year$] $album$" & @crlf & _
		"TrackDir_MP3=@OutRoot@\MP3\$cdartist$\[$year$] $album$" & @crlf & _
		"TrackDir_AAC=@OutRoot@\AAC\$cdartist$\[$year$] $album$" & @crlf & _
		"TrackDir_OGG=@OutRoot@\OGG\$cdartist$\[$year$] $album$" & @crlf & _
		"TrackName_SA=$track$ - $title$" & @crlf & _
		"TrackName_VA=$track$ - $title$ [$artist$]" & @crlf & _
		"TrackName_SA_acdir=$n - $~t" & @crlf & _
		"TrackName_VA_acdir=$n - $~t [$~a]" & @crlf & @crlf)

	FileWrite($g_iniFile, _
		"[UserSettings]" & @crlf & _
		"Debug=0" & @crlf & _
		"Comment=Created with EAC/REACT2, @curdate@" & @crlf & _
		"EmbedCover=1" & @crlf & _
		"ReplayGain=1" & @crlf & _
		"ApplyAlbumGain=1" & @crlf & _
		"AdjustAlbumGain_dB=+3.0" & @crlf & _
		"AddCuesheetAG=0" & @crlf & _
		"UseWaveGainAG=0" & @crlf & _
		"Opt_Flac=-5 -f" & @crlf & _
		"Opt_Wavpack=-hmy" & @crlf & _
		"Opt_LameMP3=-V5 --vbr-new --noreplaygain --nohist" & @crlf & _
		"Opt_NeroAac=-lc -q 0.21" & @crlf & _
		"Opt_iTunesAac=-d -s 2000" & @crlf & _
		"Opt_OggEnc2=-q 3.0" & @crlf & _
		"Ver_Flac=1.1.3" & @crlf & _
		"Ver_Wavpack=4.40" & @crlf & _
		"Ver_LameMP3=3.97" & @crlf & _
		"Ver_NeroAac=1.0.0.2" & @crlf & _
		"Ver_OggEnc2=2.83 Lancer aoTuV b5" & @crlf & @crlf)
EndFunc ; fWriteIni()


Func fHelp()
	Local $URL = "http://wiki.hydrogenaudio.org/index.php?title=REACT"
	RunWait("rundll32.exe url.dll,FileProtocolHandler " & $URL, @WorkingDir)
	;RunWait(@ComSpec & ' /c start ' & $URL, @WorkingDir, @SW_HIDE)
EndFunc

Func fHelpEAC()
	Local $URL = "http://wiki.hydrogenaudio.org/index.php?title=Exact_Audio_Copy"
	RunWait("rundll32.exe url.dll,FileProtocolHandler " & $URL, @WorkingDir)
EndFunc


Func fSetEACTitle()
	Local $t = WinGetTitle($g_eacTitle, $g_eacText)
	Local $p = StringInStr($t, "   -   REACT v", 1)
	If $p > 0 Then $t = StringLeft($t, $p - 1)
	WinSetTitle($g_eacTitle, $g_eacText, $t & "   -   REACT v" & $g_version & "   Help: F1 / Alt-F1")
EndFunc


Func fReadIni()
	$g_imageExt = IniRead($g_iniFile, "Settings", "ImageExt", $g_imageExt)
	$g_imageNaming	= IniRead($g_iniFile, "Settings", "ImageNaming", "$artist$ - $album$")
	$g_imageHotKey	= IniRead($g_iniFile, "Settings", "ImageHotKey", $g_imageHotKey)
	$g_tracksHotKey	= IniRead($g_iniFile, "Settings", "TracksHotKey", $g_tracksHotKey)
	$g_tracksHotVal	= IniRead($g_iniFile, "Settings", "TracksHotVal", $g_tracksHotVal)
	$g_runCoverDownloader = Int(IniRead($g_iniFile, "Settings", "RunCoverDownloader", $g_runCoverDownloader))
	$g_coverDownloaderExe = IniRead($g_iniFile, "Settings", "CoverDownloader", $g_coverDownloaderExe)
EndFunc


Func fShowIni()
	RunWait('"notepad.exe" "' & $g_iniFile & '"')
	fReadIni()
	fSetEACTitle()
EndFunc

Func fExploreConfig()
	RunWait('"notepad.exe" "' & @ScriptDir & '\' & $g_appName & '-image.cfg"')
EndFunc


Func fConfigEAC()
	Local $mb, $p, $v
	$mb = MsgBox(4+32+8192, $g_appTitle, "Do you want to configure EAC for use with REACT?" & @lf & _
	                                  "If you press No, you can configure EAC later by Ctrl-F2.")
	If $mb = 7 Then Return ; "NO"

	WinClose($g_eacTitle, $g_eacText)
	If WinWaitClose($g_eacTitle, $g_eacText, 5) = 0 Then
		MsgBox(0+48+8192, $g_appTitle, "Cannot close EAC")
		Return
	EndIf

	$p = "HKEY_CURRENT_USER\Software\AWSoftware\EAC\Compression Options"
	RegWrite($p, "UseExternalEncoder", "REG_BINARY", "FF")
	RegWrite($p, "ExternalEncoderType", "REG_BINARY", "14000000")
	RegWrite($p, "ExternalEncoderProgram", "REG_SZ", @ScriptFullPath)
	RegWrite($p, "ExternalEncoderOptions", "REG_SZ", 'REACT %o %s %d "%a" "%g" "%t" "%n" "%x" "%y" "%m" "%e" "%f" "%b" %r')
	RegWrite($p, "ExternalEncoderExtension", "REG_SZ", "eac")
	RegWrite($p, "ExternalEncoderCreateCRC", "REG_BINARY", "00")
	RegWrite($p, "ExternalEncoderID3Tag", "REG_BINARY", "00")
	RegWrite($p, "ExternalEncoderCheckReturnCode", "REG_BINARY", "00")
	RegWrite($p, "ExternalEncoderDeleteSource", "REG_BINARY", "00")

	$p = "HKEY_CURRENT_USER\Software\AWSoftware\EAC\Extraction Options"
	RegWrite($p, "BackgroundExternalCompression", "REG_BINARY", "FF")
	RegWrite($p, "NoCompressionWindow", "REG_BINARY", "00")
	RegWrite($p, "BeepWhenDone", "REG_BINARY", "00")
	RegWrite($p, "AutoSaveStatus", "REG_BINARY", "FF")

	$v = RegRead($p, "FileNamingConvention")
	If $v = "" Or $v = "%T" Then
		RegWrite($p, "UseVariousFileNamingConvention", "REG_BINARY", "FF")
		RegWrite($p, "OpenCDInfoDialog", "REG_BINARY", "FF")
		RegWrite($p, "RetrieveCDDBOnUnknownCD", "REG_BINARY", "FF")
	EndIf

	RegWrite($p, "FileNamingConvention", "REG_SZ", "%D - %C - %N - %T")
	RegWrite($p, "VariousFileNamingConvention", "REG_SZ", "%D - %C - %N - %T (%A)")

	MsgBox(0+64+8192, $g_appTitle, "EAC is now configured for use with REACT")
EndFunc


Func fGUISetVars()
	$gui_album = ControlGetText($g_eacTitle, $g_eacText, "myedit1")
	$gui_artist = ControlGetText($g_eacTitle, $g_eacText, "myedit2")
	$gui_year = ControlGetText($g_eacTitle, $g_eacText, "myedit3")
	$gui_genre = ControlGetText($g_eacTitle, $g_eacText, "mycombo1")
	$gui_freedb = ControlGetText($g_eacTitle, $g_eacText, "mycombo2")
	If $gui_genre = "" And $gui_freedb <> "" Then
		ControlSetText($g_eacTitle, $g_eacText, "mycombo1", $gui_freedb)
		ControlSend($g_eacTitle, $g_eacText, "mycombo1", "{ENTER}")
		$gui_genre = $gui_freedb
	EndIf

	Local $directory = RegRead("HKEY_CURRENT_USER\Software\AWSoftware\EAC\Extraction Options", "DirectorySpecification")
	Local $illeg = '"*?:<>|\/'
	Local $album = $gui_album
	For $i=1 To StringLen($illeg)
		$album = StringReplace($album, StringMid($illeg, $i, 1), ' ')
	Next
	If StringRight($directory, 1) <> '\' Then $directory = $directory + '\'
	$gui_albumfile = $directory & $album
EndFunc


Func fGUISubstVars($string)
	Local $artist_fn = fSubstIllegalChars($gui_artist, 1)
	$string = StringReplace($string, "$album$", fSubstIllegalChars($gui_album, 1))
	$string = StringReplace($string, "$cdartist$", $artist_fn)
	$string = StringReplace($string, "$artist$", $artist_fn)
	$string = StringReplace($string, "$year$", $gui_year)
	$string = StringReplace($string, "$genre$", fSubstIllegalChars($gui_genre))
	$string = StringReplace($string, "$freedb$", $gui_freedb)
	Return $string
EndFunc


Func fSubstIllegalChars($str, $stripTrailingDots = 0)
	$str = StringReplace($str, '/', $g_convTable[1], 0, 1)
	$str = StringReplace($str, '\', $g_convTable[2], 0, 1)
	$str = StringReplace($str, ':', $g_convTable[3], 0, 1)
	$str = StringReplace($str, '?', $g_convTable[4], 0, 1)
	$str = StringReplace($str, '|', $g_convTable[5], 0, 1)
	$str = StringReplace($str, '"', $g_convTable[6], 0, 1)
	$str = StringReplace($str, '*', $g_convTable[7], 0, 1)
	$str = StringReplace($str, '<', $g_convTable[8], 0, 1)
	$str = StringReplace($str, '>', $g_convTable[9], 0, 1)
	If $g_convTable[0] > 9 Then $str = StringReplace($str, ' ', $g_convTable[10], 0, 1)
	If $stripTrailingDots = 1 Then
		While StringRight($str, 1) == ' ' Or StringRight($str, 1) == '.'
			$str = StringTrimRight($str, 1)
		Wend
	EndIf
	Return $str
EndFunc


Func fGUIRunCoverDownloader()
	fGUIRunWaitCoverDownloader(0)
EndFunc


Func fGUIRunWaitCoverDownloader($wait)
	fGUISetVars()
	Local $cmd = '"' & $g_coverDownloaderExe & '" "' & $gui_artist & '" "' & $gui_album & '" "' & $gui_albumfile & '.jpg"'
	IF $wait Then
		RunWait($cmd)
	Else
		Run($cmd)
	EndIf
EndFunc


Func fGUICreateCuesheets()
	Local $orgcuesheet = $gui_albumfile & ".CUE"

	ControlSend($g_eacTitle, $g_eacText, "", "!s")
	fMoveCuesheet($orgcuesheet, $gui_albumfile & ".[s].cue")
	WinMenuSelectItem($g_eacTitle, $g_eacText, $g_cueMenu[1], $g_cueMenu[2], $g_cueMenu[3])
	fMoveCuesheet($orgcuesheet, $gui_albumfile & ".[mg].cue")
	ControlSend($g_eacTitle, $g_eacText, "", "!m")
	fMoveCuesheet($orgcuesheet, $gui_albumfile & ".[m].cue")
	ControlSend($g_eacTitle, $g_eacText, "", "!o")
	fMoveCuesheet($orgcuesheet, $gui_albumfile & ".[mc].cue")
EndFunc


Func fCreateCuesheetsTest()
	If Int(IniRead($g_iniFile, "Settings", "CreateAllCuesheets", 0)) And _
	   RegRead("HKEY_CURRENT_USER\Software\AWSoftware\EAC\Extraction Options", "DirectoryUse") = "00000000" Then
		Return 1
	EndIf
	Return 0
EndFunc


Func fMoveCuesheet($src, $dest)
	Do
		Sleep(300)
	Until FileExists($src)
	FileMove($src, $dest, 1)
EndFunc


Func fTracksCopyCompressed()
	fGUISetVars()

	FileDelete($gui_albumfile & '.log')
	If fCreateCuesheetsTest() Then fGUICreateCuesheets()
	If $g_runCoverDownloader Then fGUIRunWaitCoverDownloader(BitAnd($g_runCoverDownloader, 1))

	ControlSend($g_eacTitle, $g_eacText, "", $g_tracksHotVal)
EndFunc


Func fImageCopyCompressed()
	fGUISetVars()

	FileDelete($gui_albumfile & '.log')
	If fCreateCuesheetsTest() Then fGUICreateCuesheets()
	If $g_runCoverDownloader Then fGUIRunWaitCoverDownloader(BitAnd($g_runCoverDownloader, 2))

	Local $name = fGUISubstVars($g_imageNaming)
	WinMenuSelectItem($g_eacTitle, $g_eacText, $g_imgMenu[1], $g_imgMenu[2], $g_imgMenu[3])
	WinWait($g_saveWaveformStr, "", 5)
	ControlSetText($g_saveWaveformStr, "", "Edit1", $name)
	; Do it again after 0.2s in case it failed...
	Sleep(250)
	ControlSetText($g_saveWaveformStr, "", "Edit1", $name)

	If RegRead("HKEY_CURRENT_USER\Software\AWSoftware\EAC\Extraction Options", "DirectoryUse") = "00000000" Then
		Sleep(250)
		ControlSend($g_saveWaveformStr, "", "Edit1", "{ENTER}")
	EndIf
EndFunc



#cs  EAC
  1  <config>
  2  %o        original filename (no dir, and without temporary renaming)
  3  %s        temporary source filename
  4  %d        temporary destination filename
  5  %a        @artist@
  6  %g        @album@
  7  %t        @title@
  8  %n        @track@
  9  %x        @numtracks@
  10 %y        @year@
  11 %m        @genre@
  12 %e        @comment@
  13 %f        @freedbid@
  14 %b        @crc@
  15 %r        @bitrate@
#ce

Func fEncode()
	Local $a, $val, $temp_root, $temp_ext, $hasCue, $cuesheet_org
	Local $batdata, $pid, $winState
	Local $cuedata_org, $cuedata, $configFile, $basepath
	Local $cuesheetExt = "cue"

	; Determine temp compression extension - normally .eac
	$a = StringSplit($CmdLine[4], ".")
	$temp_ext = $a[$a[0]]
	$temp_root = StringTrimRight($CmdLine[4], StringLen($temp_ext) + 1) & "-react"

	; Determine source/destination directory
	$a = StringSplit($CmdLine[4], "\")
	$g_sourcedir = StringTrimRight($CmdLine[4], StringLen($a[$a[0]]) + 1) ; remove tmp name
	$g_basename = StringTrimRight($CmdLine[2], 4) ; remove ".wav" ext.
	$basepath = $g_sourcedir & "\" & $g_basename

	; The real source .wav
	$g_source_tmp = $CmdLine[3]
	$g_artist = StringReplace($CmdLine[5], '"', "'", 0, 1)
	$g_album = StringReplace($CmdLine[6], '"', "'", 0, 1)
	$g_title = StringReplace($CmdLine[7], '"', "'", 0, 1)
	$g_track = Int($CmdLine[8]) ; force to number
	$g_numtracks = Int($CmdLine[9])
	$g_year = $CmdLine[10]
	$g_genre = $CmdLine[11]
	$g_comment = StringReplace($CmdLine[12], '"', "'", 0, 1)
	$g_freedbID = $CmdLine[13]
	$g_crc = $CmdLine[14]
	$g_bitrate = $CmdLine[15]

	; This block is a workaround for a bug in parameter parsing in AutoIt3.1.1
	If $g_artist = "'" Then $g_artist = ""
	If $g_album = "'" Then $g_album = ""
	If $g_title = "'" Then $g_title = ""
	If $g_year = '"' Then $g_year = ""
	If $g_genre = '"' Then $g_genre = ""
	If $g_comment = "'" Then $g_comment = ""

	$g_artist_fn = fSubstIllegalChars($g_artist, 1)
	$g_album_fn = fSubstIllegalChars($g_album, 1)
	$g_title_fn = fSubstIllegalChars($g_title)
	$g_track_fn = StringFormat("%02d", $g_track) ; force to number
	$g_numtracks_fn = StringFormat("%02d", $g_numtracks) ; force to number
	$g_genre_fn = fSubstIllegalChars($g_genre)
	$g_comment_fn = fSubstIllegalChars($g_comment)

	$g_VA = IniRead($g_iniFile, "Settings", "VA", $g_VA) ; Various Artists

	If StringLeft($CmdLine[2], 9) = "Various -" Or _
	   StringLeft($CmdLine[2], 17) = "Various Artists -" Then
		$g_cdartist = $g_VA
		$g_cdartist_fn = $g_VA
		$g_various = 1
	Else
		$g_cdartist = $g_artist
		$g_cdartist_fn = $g_artist_fn
	EndIf

	$g_toolsDir = FileGetShortName(IniRead($g_iniFile, "Settings", "Tools", $g_toolsDir))
	$g_processPriority = IniRead($g_iniFile, "Settings", "ProcessPriority", $g_processPriority)
	$g_albumfile = $g_album
	Local $illeg = '"*?:<>|\/'
	For $i=1 To StringLen($illeg)
		$g_albumfile = StringReplace($g_albumfile, StringMid($illeg, $i, 1), ' ')
	Next
	$g_albumfile = $g_sourcedir & "\" & $g_albumfile

	$g_eaclog = $g_albumfile & ".log"
	$g_cover = $g_albumfile & ".jpg"

	$g_userTrackFormats = IniReadSection($g_iniFile, "UserTrackFormats")
	$g_userOutputNames = IniReadSection($g_iniFile, "UserOutputNames")
	$g_userSettings = IniReadSection($g_iniFile, "UserSettings")

	; -----------

	; Get image extension from ini-file
	$g_imageExt = IniRead($g_iniFile, "Settings", "ImageExt", $g_imageExt)

	; Check if a cuesheet was created ==> cd image.

	$cuesheet_org = $basepath & ".cue"
	$hasCue = FileExists($cuesheet_org)
	If Not $hasCue Then
		$cuesheet_org = $basepath & "." & $temp_ext & ".cue"
		$hasCue = FileExists($cuesheet_org)
	EndIf

	If $hasCue Then
		$configFile = @ScriptDir & '\' & $g_appName & '-image.cfg'
	Else
		$configFile = @ScriptDir & '\' & $g_appName & '-track.cfg'
	EndIf

	If Not FileExists($configFile) Then
		MsgBox(0+48+8192, $g_appTitle, "No configuration file found: " & $configFile, 5)
		Return
	EndIf


	If $hasCue Then
		$cuedata_org = FileRead($cuesheet_org, FileGetSize($cuesheet_org))

		; --------- Set the temp sourcecuesheet -----------
		Local $c1 = StringSplit($cuedata_org, 'FILE "', 1)
		Local $c2 = StringSplit($cuedata_org, '" WAVE', 1)
		$a = StringSplit($g_source_tmp, "\")
		$cuedata = $c1[1] & 'FILE "' & $a[$a[0]] & '" WAVE' & $c2[2]

		$g_sourcecuesheet = $temp_root & ".cue"
		FileDelete($g_sourcecuesheet)
		FileWrite($g_sourcecuesheet, $cuedata)

		; --------- Set the destination cuesheet -----------
		Local $cueDataType = 'WAVE'
		If $g_imageExt = "mp3" Then $cueDataType = 'MP3'

		$g_destimage = $basepath & "." & $g_imageExt
		$g_cuesheet = $basepath & "." & $cuesheetExt

		$cuedata = StringReplace($cuedata_org, '.' & $temp_ext & '" WAVE', '.' & $g_imageExt & '" ' & $cueDataType)
	EndIf

	; Determine windows state to set
	If RegRead("HKEY_CURRENT_USER\Software\AWSoftware\EAC\Extraction Options", "NoCompressionWindow") = "FF" Then
		$winState = @SW_HIDE
	ElseIf Int(IniRead($g_iniFile, "Settings", "MinimizeCompressionWindow", $g_minCompWindow)) Then
		$winState = @SW_MINIMIZE
	Else
		$winState = @SW_SHOW
	EndIf

	; Create the final cuesheet.
	If $hasCue Then
		FileDelete($cuesheet_org)
		FileDelete($g_cuesheet)
		; Save the new .cue file
		FileWrite($g_cuesheet, $cuedata)
	EndIf

	$g_batfile = $g_albumfile & ".bat"

	; Read the config file
	Local $cmd = FileRead($configFile, FileGetSize($configFile))

	; Create the .bat file
	$batdata = fSubstVars($cmd)

	FileDelete($g_batfile)
	FileWrite($g_batfile, $batdata)

	; Make sure the EAC log file is written before continuing - we want to embed it in image
	If $hasCue Or $g_track = $g_numtracks Then
		For $i=1 To 20 ; max 5 seconds
			If FileExists($g_eaclog) Then ExitLoop
			Sleep(250)
		Next
	Else
		Sleep(500)
	EndIf

	; Finally, execute the .bat file
	$pid = Run('"' & $g_batfile & '"', $g_sourcedir, $winState)
	ProcessSetPriority($pid, $g_processPriority)
	ProcessWaitClose($pid)

	; Remove temporay files
	FileDelete($g_sourcecuesheet)

	Sleep(200)
EndFunc


Func fSubstVars($str)
	For $i = $g_userSettings[0][0] To 1 Step -1
		$str = StringReplace($str, '@' & $g_userSettings[$i][0] & '@', $g_userSettings[$i][1])
	Next
	For $i = $g_userOutputNames[0][0] To 1 Step -1
		$str = StringReplace($str, '@' & $g_userOutputNames[$i][0] & '@', $g_userOutputNames[$i][1])
	Next
	For $i = $g_userTrackFormats[0][0] To 1 Step -1
		$str = StringReplace($str, '@' & $g_userTrackFormats[$i][0] & '@', $g_userTrackFormats[$i][1])
	Next
	$str = StringReplace($str, "$cdartist$", $g_cdartist_fn)
	$str = StringReplace($str, "$artist$", $g_artist_fn)
	$str = StringReplace($str, "$album$", $g_album_fn)
	$str = StringReplace($str, "$track$", $g_track_fn)
	$str = StringReplace($str, "$numtracks$", $g_numtracks_fn)
	$str = StringReplace($str, "$title$", $g_title_fn)
	$str = StringReplace($str, "$year$", $g_year)
	$str = StringReplace($str, "$genre$", $g_genre_fn)
	$str = StringReplace($str, "$comment$", $g_comment_fn)

	$str = StringReplace($str, "@cdartist@", $g_cdartist)
	$str = StringReplace($str, "@artist@", $g_artist)
	$str = StringReplace($str, "@album@", $g_album)
	$str = StringReplace($str, "@title@", $g_title)
	$str = StringReplace($str, "@track@", $g_track)
	$str = StringReplace($str, "@numtracks@", $g_numtracks)
	$str = StringReplace($str, "@year@", $g_year)
	$str = StringReplace($str, "@genre@", $g_genre)
	$str = StringReplace($str, "@freedbid@", $g_freedbID)
	$str = StringReplace($str, "@comment@", $g_comment)

	$str = StringReplace($str, "@source@", $g_source_tmp)
	$str = StringReplace($str, "@image@", $g_destimage)
	$str = StringReplace($str, "@imageext@", $g_imageExt)
	$str = StringReplace($str, "@cuesheet@", $g_cuesheet)
	$str = StringReplace($str, "@sourcecuesheet@", $g_sourcecuesheet)
	$str = StringReplace($str, "@various@", $g_various)
	$str = StringReplace($str, "@va@", $g_VA)

	$str = StringReplace($str, "@crc@", $g_crc)
	$str = StringReplace($str, "@bitrate@", $g_bitrate)
	$str = StringReplace($str, "@sourcedir@", $g_sourcedir)
	$str = StringReplace($str, "@basename@", $g_basename)
	$str = StringReplace($str, "@albumfile@", $g_albumfile)
	$str = StringReplace($str, "@cover@", $g_cover)
	$str = StringReplace($str, "@eaclog@", $g_eaclog)
	$str = StringReplace($str, "@tools@", $g_toolsDir)
	$str = StringReplace($str, "@reactdir@", @ScriptDir)
	$str = StringReplace($str, "@mymusic@", $g_myMusicDir)
	$str = StringReplace($str, "@curdate@", @YEAR &'-'& @MON &'-'& @MDAY)

	Return $str
EndFunc
